#!/bin/sh
# Script to summarize buildlogs/$VER/*.status.txt as buildlogs/$VER/index.html

set -x

# hmm, we should figure out a way to avoid having to manually change
# the crosstool number in four files on each release...
VER=0.28

# Figure out which tool combinations were used

ALL_TOOLS=`awk -F= '/toolcombo/ { print $2 }' buildlogs/$VER/*.status.txt | sort -u`
ALL_CPUS=`cd buildlogs/$VER; ls *.status.txt | sed 's/-gcc-.*.status.txt//' | sort -u`

OUT=buildlogs/$VER/index.html
cat  > $OUT <<_EOF_
<html>
<head>
<title>Crosstool build results</title>
</head>
<body>
<h2>Crosstool build results</h2>
The following matrix shows whether the given combinations of
gcc, glibc, binutils, and linux kernel headers, lightly patched,
can build a cross-toolchain and compile a kernel for the given CPUs. 
(It doesn't say anything about whether the resulting toolchain works!)
It was generated by running the script demo-loop.sh that comes with 
<a href="http://kegel.com/crosstool">crosstool</a>.
Click on a matrix entry for the last few lines of that build log file.
_EOF_

echo "<table><tr><th>" >> $OUT

for toolcombo in $ALL_TOOLS; do
      binutils=`awk -F= '/BINUTILS_DIR/ {print $2}' $toolcombo.dat`
      linux=`awk -F= '/LINUX_DIR/ {print $2}' $toolcombo.dat`
      echo "<th>$toolcombo<br>$binutils<br>$linux<br></th>" | sed "s/-glibc/<br>glibc/" >> $OUT
done
echo '</tr>' >> $OUT

for cpu in $ALL_CPUS; do
   echo '<tr><th>'$cpu'</th>'
   for toolcombo in $ALL_TOOLS; do
      echo '<td ' 
      if test -f buildlogs/$VER/$cpu-$toolcombo.status.txt; then
        status=`cat buildlogs/$VER/$cpu-$toolcombo.status.txt`
        case "$status" in
        *kernel=PASS*)    echo 'style="background-color: green"><a href="'$cpu-$toolcombo.log.txt'">ok</a>'           ;;
        *toolchain=PASS*) echo 'style="background-color: yellow"><a href="'$cpu-$toolcombo.log.txt'">kernel fail</a>' ;;
        *toolchain=ICE*)  echo 'style="background-color: orange"><a href="'$cpu-$toolcombo.log.txt'">ICE</a>'         ;;
        *toolchain=FAIL*) echo 'style="background-color: red"><a href="'$cpu-$toolcombo.log.txt'">FAIL</a>'           ;;
	*) echo 'bug in demo-loop' ;;
        esac
      fi
      echo '</td>' 
   done
   echo '</tr>' 
done >> $OUT
echo '</table>' >> $OUT
echo '</body></html>' >> $OUT

#sed 's,href=",href="../buildlogs/'$VER'/,' < $OUT > results.html
