#!/bin/sh
# Script to build a large number of variations of the toolchain, and create
# a .html file describing whether each build succeeded.

set -x
TARBALLS_DIR=$HOME/downloads
RESULT_TOP=/opt/crosstool
export TARBALLS_DIR RESULT_TOP
GCC_LANGUAGES="c,c++"
export GCC_LANGUAGES

# We grep through a huge logfile, and on Red Hat 9, that's slow unless you use C locale
LANG=C
export LANG

# Which toolcombos to test.  
# I don't think anyone's using 3.0 or 3.2 for new development.
# We test 2.95.3 because some people are stuck with it,
# 3.3.4 because it's the 'conservative' choice,
# and 3.4.2 because it's the latest release.
# We might add 2.96-rh sometime simply because it's the production compiler on RHAS.
ALL_TOOLS="\
gcc-2.95.3-glibc-2.1.3 \
gcc-2.95.3-glibc-2.2.5 \
gcc-3.3.4-glibc-2.2.5 \
gcc-3.3.4-glibc-2.3.2 \
gcc-3.4.2-glibc-2.2.5 \
gcc-3.4.2-glibc-2.3.3 \
gcc-3.4.2-glibc-20040827 \
"
TOOLS="$ALL_TOOLS"
#TOOLS="gcc-3.3.4-glibc-2.3.3 gcc-3.4.2-glibc-2.3.3"

# What version of crosstool this is
VER=0.28

# ignore cris and hppa for now, they're broken until the maintainers
# submit glibc updates
ALL_CPUS=`ls *.dat | grep -v gcc | grep -v cygwin | egrep -v 'cris|hppa' | sed 's/\.dat$//' `
CPUS="$ALL_CPUS"

# Please read doc/crosstool-howto.html

# Set this if you have a dual CPU build machine, or are using distcc
#PARALLELMFLAGS="-j2"
#export PARALLELMFLAGS

# Really, you should do the mkdir before running this,
# and chown /opt/crosstool to yourself so you don't need to run as root.
mkdir -p $RESULT_TOP

mkdir -p buildlogs/$VER

if true; then
 for cpu in $CPUS; do
   for toolcombo in $TOOLS; do
       if true; then
	 # remove temp files and even resulting toolchain to avoid filling up disk
	 # Do it at top of loop so last one we build hangs around for debugging.  FIXME
	 rm -rf build /opt/crosstool/*

	 # use a subshell so environment variables don't leak out,
	 # as they are rumored to with some shells
	 ( eval `cat $cpu.dat $toolcombo.dat` sh all.sh --notest --testlinux > $cpu-$toolcombo.log 2>&1 )
       fi
       tail -200 $cpu-$toolcombo.log > buildlogs/$VER/$cpu-$toolcombo.log.txt
       if tail buildlogs/$VER/$cpu-$toolcombo.log.txt | grep "Linux kernel build done" > /dev/null; then
          tresult="PASS"
          kresult="PASS"
       elif grep "internal compiler error" $cpu-$toolcombo.log > /dev/null; then
          tresult="ICE"
          kresult="FAIL"
       elif grep "testhello: C compiler can in fact build a trivial program" $cpu-$toolcombo.log > /dev/null; then
          tresult="PASS"
          kresult="FAIL"
       else
          tresult="FAIL"
          kresult="FAIL"
       fi
       target=`awk -F= '/TARGET/ {print $2}' $cpu.dat`
       binutils=`awk -F= '/BINUTILS_DIR/ {print $2}' $toolcombo.dat`
       linux=`awk -F= '/LINUX_DIR/ {print $2}' $toolcombo.dat`
       printf "toolcombo=$toolcombo\nbinutils=$binutils\nlinux=$linux\ntarget=$target\ntoolchain=$tresult\nkernel=$kresult\n" > buildlogs/$VER/$cpu-$toolcombo.status.txt
   done
done
fi

# Eventually, following section will be driven not by a loop over
# CPUS and TOOLS, but by a loop over *.status.txt files, and will
# be in a separate script, so you can run separately from running tests.
OUT=buildlogs/$VER/index.html
cat  > $OUT <<_EOF_
<html>
<head>
<title>Crosstool build results</title>
</head>
<body>
<h2>Crosstool build results</h2>
The following matrix shows whether the given combinations of
gcc, glibc, binutils, and linux kernel headers, lightly patched,
can build a cross-toolchain and compile a kernel for the given CPUs. 
(It doesn't say anything about whether the resulting toolchain works!)
It was generated by running the script demo-loop.sh that comes with 
<a href="http://kegel.com/crosstool">crosstool</a>.
Click on a matrix entry for the last few lines of that build log file.
_EOF_

echo "<table><tr><th>" >> $OUT

for toolcombo in $ALL_TOOLS; do
      binutils=`awk -F= '/BINUTILS_DIR/ {print $2}' $toolcombo.dat`
      linux=`awk -F= '/LINUX_DIR/ {print $2}' $toolcombo.dat`
      echo "<th>$toolcombo<br>$binutils<br>$linux<br></th>" | sed "s/-glibc/<br>glibc/" >> $OUT
done
echo '</tr>' >> $OUT

for cpu in $ALL_CPUS; do
   echo '<tr><th>'$cpu'</th>'
   for toolcombo in $ALL_TOOLS; do
      echo '<td ' 
      if test -f buildlogs/$VER/$cpu-$toolcombo.status.txt; then
        status=`cat buildlogs/$VER/$cpu-$toolcombo.status.txt`
        case "$status" in
        *kernel=PASS*)    echo 'style="background-color: green"><a href="'$cpu-$toolcombo.log.txt'">ok</a>'           ;;
        *toolchain=PASS*) echo 'style="background-color: yellow"><a href="'$cpu-$toolcombo.log.txt'">kernel fail</a>' ;;
        *toolchain=ICE*)  echo 'style="background-color: orange"><a href="'$cpu-$toolcombo.log.txt'">ICE</a>'         ;;
        *toolchain=FAIL*) echo 'style="background-color: red"><a href="'$cpu-$toolcombo.log.txt'">FAIL</a>'           ;;
	*) echo 'bug in demo-loop' ;;
        esac
      fi
      echo '</td>' 
   done
   echo '</tr>' 
done >> $OUT
echo '</table>' >> $OUT
echo '</body></html>' >> $OUT

#sed 's,href=",href="../buildlogs/'$VER'/,' < $OUT > results.html
